==> JDFLibJ-2.1.4a.76-sources.jar: org/cip4/jdflib/elementwalker/ElementWalker.java <==
/*
 * The CIP4 Software License, Version 1.0
 *
 *
 * Copyright (c) 2001-2011 The International Cooperation for the Integration of
 * Processes in  Prepress, Press and Postpress (CIP4).  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        The International Cooperation for the Integration of
 *        Processes in  Prepress, Press and Postpress (www.cip4.org)"
 *    Alternately, this acknowledgment mrSubRefay appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "CIP4" and "The International Cooperation for the Integration of
 *    Processes in  Prepress, Press and Postpress" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact info@cip4.org.
 *
 * 5. Products derived from this software may not be called "CIP4",
 *    nor may "CIP4" appear in their name, without prior writtenrestartProcesses()
 *    permission of the CIP4 organization
 *
 * Usage of this software in commercial products is subject to restrictions. For
 * details please consult info@cip4.org.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE INTERNATIONAL COOPERATION FOR
 * THE INTEGRATION OF PROCESSES IN PREPRESS, PRESS AND POSTPRESS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIrSubRefAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the The International Cooperation for the Integration
 * of Processes in Prepress, Press and Postpress and was
 * originally based on software restartProcesses()
 * copyright (c) 1999-2001, Heidelberger Druckmaschinen AG
 * copyright (c) 1999-2001, Agfa-Gevaert N.V.
 *
 * For more information on The International Cooperation for the
 * Integration of Processes in  Prepress, Press and Postpress , please see
 * <http://www.cip4.org/>.
 *
 */
package org.cip4.jdflib.elementwalker;

import org.cip4.jdflib.core.KElement;
import org.cip4.jdflib.core.VElement;

/**
 * 
 * elementwalker class that allows you to traverse a dom tree starting at a given root
 * 
 * @author prosirai
 * 
 */
public class ElementWalker
{
	protected IWalkerFactory theFactory;

	/**
	 * @param _theFactory used to find the individual instances for the children
	 */
	public ElementWalker(final IWalkerFactory _theFactory)
	{
		super();
		this.theFactory = _theFactory;
	}

	/**
	 * walk the tree starting at e.
	 * 
	 * @param e the root element to walk, must not be null
	 * @param trackElem a parallel element to e that may additionally be modified during the walk, may be null
	 * @return n the number of traversed elements
	 */
	public int walkTree(final KElement e, final KElement trackElem)
	{
		if (e == null)
		{
			return 0;
		}
		int n = 0;

		final IWalker w = theFactory.getWalker(e);
		KElement b = null;
		if (w != null)
		{
			n++;
			b = doWalk(e, trackElem, w);
		}
		if (b != null) // follow kids if still alive or no walker found
		{
			// do not follow refelements
			final VElement v = e.getChildElementVector_KElement(null, null, null, true, -1);
			for (KElement e2 : v)
			{
				n += walkTree(e2, b);
			}
		}
		return n;
	}

	/**
	 * @param e
	 * @param trackElem
	 * @param w
	 * @return
	 */
	private KElement doWalk(final KElement e, final KElement trackElem, final IWalker w)
	{
		KElement b = null;
		try
		{
			w.prepareWalk(e, trackElem);
		}
		finally
		{
			// nop
		}
		try
		{
			b = w.walk(e, trackElem);
		}
		finally
		{
			// nop
		}
		try
		{
			w.finalizeWalk(e, trackElem);
		}
		finally
		{
			// nop
		}
		return b;
	}

	/**
	 * walk the tree starting at e.
	 * 
	 * @param e the root element to walk
	 * @return n the number of traversed elements
	 */
	public int walkTreeKidsFirst(final KElement e)
	{
		if (e == null)
		{
			return 0;
		}
		int n = 0;
		// do not follow refelements - also MUST use VElement rather than iterator to retain walking ability, even if elements are deleted
		final VElement v = e.getChildElementVector_KElement(null, null, null, true, -1);
		for (KElement e2 : v)
		{
			n += walkTreeKidsFirst(e2);
		}

		final IWalker w = theFactory.getWalker(e);
		if (w != null)
		{
			n++;
			doWalk(e, null, w);
		}
		return n;
	}

	/**
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return getClass().getSimpleName() + " : " + theFactory;
	}
}

==> JDFLibJ-2.1.5.86-sources.jar: org/cip4/jdflib/elementwalker/ElementWalker.java <==
/*
 * The CIP4 Software License, Version 1.0
 *
 *
 * Copyright (c) 2001-2014 The International Cooperation for the Integration of
 * Processes in  Prepress, Press and Postpress (CIP4).  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        The International Cooperation for the Integration of
 *        Processes in  Prepress, Press and Postpress (www.cip4.org)"
 *    Alternately, this acknowledgment mrSubRefay appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "CIP4" and "The International Cooperation for the Integration of
 *    Processes in  Prepress, Press and Postpress" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact info@cip4.org.
 *
 * 5. Products derived from this software may not be called "CIP4",
 *    nor may "CIP4" appear in their name, without prior writtenrestartProcesses()
 *    permission of the CIP4 organization
 *
 * Usage of this software in commercial products is subject to restrictions. For
 * details please consult info@cip4.org.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE INTERNATIONAL COOPERATION FOR
 * THE INTEGRATION OF PROCESSES IN PREPRESS, PRESS AND POSTPRESS OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIrSubRefAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the The International Cooperation for the Integration
 * of Processes in Prepress, Press and Postpress and was
 * originally based on software restartProcesses()
 * copyright (c) 1999-2001, Heidelberger Druckmaschinen AG
 * copyright (c) 1999-2001, Agfa-Gevaert N.V.
 *
 * For more information on The International Cooperation for the
 * Integration of Processes in  Prepress, Press and Postpress , please see
 * <http://www.cip4.org/>.
 *
 */
package org.cip4.jdflib.elementwalker;

import org.cip4.jdflib.core.KElement;
import org.cip4.jdflib.core.VElement;

/**
 * 
 * elementwalker class that allows you to traverse a dom tree starting at a given root
 * 
 * @author prosirai
 * 
 */
public class ElementWalker
{
	protected IWalkerFactory theFactory;

	/**
	 * @param _theFactory used to find the individual instances for the children
	 */
	public ElementWalker(final IWalkerFactory _theFactory)
	{
		super();
		this.theFactory = _theFactory;
	}

	/**
	 * walk the tree starting at e.
	 * 
	 * @param e the root element to walk, must not be null
	 * @param trackElem a parallel element to e that may additionally be modified during the walk, may be null
	 * @return n the number of traversed elements
	 */
	public int walkTree(final KElement e, final KElement trackElem)
	{
		if (e == null)
		{
			return 0;
		}
		int n = 0;

		final IWalker w = theFactory.getWalker(e);
		KElement b = null;
		if (w != null)
		{
			n++;
			b = doWalk(e, trackElem, w);
		}
		if (b != null) // follow kids if still alive or no walker found
		{
			// do not follow refelements
			final VElement v = e.getChildElementVector_KElement(null, null, null, true, -1);
			for (KElement e2 : v)
			{
				n += walkTree(e2, b);
			}
		}
		return n;
	}

	/**
	 * @param e
	 * @param trackElem
	 * @param w
	 * @return
	 */
	private KElement doWalk(final KElement e, final KElement trackElem, final IWalker w)
	{
		KElement b = null;
		try
		{
			w.prepareWalk(e, trackElem);
		}
		finally
		{
			// nop
		}
		try
		{
			b = w.walk(e, trackElem);
		}
		finally
		{
			// nop
		}
		try
		{
			w.finalizeWalk(e, trackElem);
		}
		finally
		{
			// nop
		}
		return b;
	}

	/**
	 * walk the tree starting at e.
	 * 
	 * @param e the root element to walk
	 * @return n the number of traversed elements
	 */
	public int walkTreeKidsFirst(final KElement e)
	{
		if (e == null)
		{
			return 0;
		}
		int n = 0;
		// do not follow refelements - also MUST use VElement rather than iterator to retain walking ability, even if elements are deleted
		final VElement v = e.getChildElementVector_KElement(null, null, null, true, -1);
		for (KElement e2 : v)
		{
			n += walkTreeKidsFirst(e2);
		}

		final IWalker w = theFactory.getWalker(e);
		if (w != null)
		{
			n++;
			doWalk(e, null, w);
		}
		return n;
	}

	/**
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return getClass().getSimpleName() + " : " + theFactory;
	}

	/**
	 * Getter for theFactory attribute.
	 * @return the theFactory
	 */
	public IWalkerFactory getFactory()
	{
		return theFactory;
	}
}
